-- TODO: This variable should be obtained from the system ENV
\set DB dnet_wfs
CREATE DATABASE :DB;
\c :DB

BEGIN;

-- WF History

CREATE TABLE wf_journal (
	process_id     text PRIMARY KEY,
	wf_template_id text NOT NULL,
	wf_conf_id     text,
	wf_executor    text,
	name           text NOT NULL,
	family         text NOT NULL,
	status         text NOT NULL,
	start_date     timestamp,
	end_date       timestamp,
	last_update    timestamp DEFAULT now(),
	ds_id          text,
	ds_name        text,
	ds_api         text,
	details        jsonb
);

-- Workflows

CREATE TABLE wf_sections (
	id                      text PRIMARY KEY,
	name                    text NOT NULL
);

INSERT INTO wf_sections(id, name) VALUES
('GC', 'Garbage Collection'),
('CONSISTENCY', 'InfoSpace Consistency'),
('DEDUP', 'InfoSpace Deduplication'),
('INFERENCE', 'InfoSpace Inference'),
('MONITOR', 'InfoSpace Monitoring'),
('PROVISION', 'InfoSpace Provision'),
('IS', 'Information Service'),
('BROKER', 'Notification Broker');

CREATE TABLE wf_configurations (
	id                      text PRIMARY KEY,
	name                    text NOT NULL,
	section                 text REFERENCES wf_sections(id),
	details                 jsonb NOT NULL DEFAULT '{}',
	priority                int,
	dsid                    text,
	dsname                  text,
	apiid                   text,
	enabled                 boolean NOT NULL DEFAULT false,
	configured              boolean NOT NULL DEFAULT false,
	scheduling_enabled      boolean NOT NULL DEFAULT false, 
	scheduling_cron         text, 
	scheduling_min_interval int,
	workflow                text,
	destroy_wf              text,
	system_params           jsonb NOT NULL DEFAULT '{}',
	user_params             jsonb NOT NULL DEFAULT '{}'
);

CREATE TABLE wf_subscriptions (
	wf_conf_id              text NOT NULL REFERENCES wf_configurations(id),
	condition               text NOT NULL,
	email                   text NOT NULL,
	message_id              text NOT NULL,
	PRIMARY KEY (wf_conf_id, condition, email)
);

CREATE VIEW wf_sections_view AS SELECT
       s.id       AS id,
       s.name     AS name,
       count(c.*) AS count
FROM wf_sections s LEFT OUTER JOIN wf_configurations c ON (s.id = c.section)
GROUP BY s.id, s.name
ORDER BY s.name;

COMMIT;
