import { AfterViewInit, Component, Inject, OnInit, ViewChild } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { MatDialog, MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { MatSort } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';
import { ActivatedRoute } from '@angular/router';
import { Injectable } from '@angular/core';
import { EmailTemplate } from '../common/is.model';
import { ISClient } from '../common/is.client';

@Component({
	selector: 'app-emails',
	templateUrl: './emails.component.html',
	styleUrls: []
})
export class EmailsComponent implements OnInit, AfterViewInit {
	emailsDatasource: MatTableDataSource<EmailTemplate> = new MatTableDataSource<EmailTemplate>([]);

	colums: string[] = ['id', 'description', 'buttons'];

	@ViewChild(MatSort) sort: MatSort | undefined

	searchText: string = '';

	constructor(public client: EmailsClient, public route: ActivatedRoute, public dialog: MatDialog) { }

	ngOnInit() { this.reload() }
	ngAfterViewInit() { if (this.sort) this.emailsDatasource.sort = this.sort; }
	reload() { this.client.loadEmailTemplates((data: EmailTemplate[]) => this.emailsDatasource.data = data); }

	applyFilter(event: Event) {
		const filterValue = (event.target as HTMLInputElement).value.trim().toLowerCase();
		this.emailsDatasource.filter = filterValue;
	}

	openAddEmailTemplateDialog(): void {
		const dialogRef = this.dialog.open(EmailDialog, {
			data: {
				id: '',
				description: '',
				subject: '',
				message: ''
			},
			width: '80%'
		});

		dialogRef.afterClosed().subscribe(result => {
			if (result) this.reload();
		});
	}

	openEditEmailTemplateDialog(email: EmailTemplate): void {
		const dialogRef = this.dialog.open(EmailDialog, {
			data: email,
			width: '80%'
		});

		dialogRef.afterClosed().subscribe(result => {
			if (result) this.reload();
		});
	}

	deleteEmailTemplate(email: EmailTemplate) {
		if (confirm('Are you sure?')) {
			this.client.deleteEmailTemplate(email.id, (data: void) => this.reload());
		}
	}
}

@Component({
	selector: 'email-dialog',
	templateUrl: './email-dialog.html',
	styleUrls: []
})
export class EmailDialog {

	emailForm = new FormGroup({
		id: new FormControl(''),
		description: new FormControl('', [Validators.required]),
		subject: new FormControl('', [Validators.required]),
		message: new FormControl('', [Validators.required])
	});

	constructor(public dialogRef: MatDialogRef<EmailDialog>, @Inject(MAT_DIALOG_DATA) public data: any, public client: EmailsClient) {
		this.emailForm.get('id')?.setValue(data.id);
		this.emailForm.get('description')?.setValue(data.description);
		this.emailForm.get('subject')?.setValue(data.subject);
		this.emailForm.get('message')?.setValue(data.message);
	}

	onSubmit(): void {
		const email = Object.assign({}, this.data, this.emailForm.value);
		this.client.saveEmailTemplate(email, (data: void) => this.dialogRef.close(1), this.emailForm);
	}

	onNoClick(): void {
		this.dialogRef.close();
	}
}

@Injectable({
	providedIn: 'root'
})
export class EmailsClient extends ISClient {

	loadEmailTemplates(onSuccess: Function): void {
		this.httpGet<EmailTemplate[]>('/proxy/byType/templates/email/', onSuccess);
	}

	saveEmailTemplate(email: EmailTemplate, onSuccess: Function, relatedForm?: FormGroup): void {
		this.httpPost('/proxy/byType/templates/email/', email, onSuccess, relatedForm);
	}

	deleteEmailTemplate(id: string, onSuccess: Function): void {
		this.httpDelete('/proxy/byType/templates/email/' + encodeURIComponent(id), onSuccess);
	}
}
