package eu.dnetlib.domain.wfs;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.Map;

import org.hibernate.annotations.Type;

import io.hypersistence.utils.hibernate.type.json.JsonType;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.EnumType;
import jakarta.persistence.Enumerated;
import jakarta.persistence.Id;
import jakarta.persistence.PrePersist;
import jakarta.persistence.PreUpdate;
import jakarta.persistence.Table;

@Entity
@Table(name = "wf_journal")
public class WfJournalEntry implements Serializable {

	private static final long serialVersionUID = -326994850248506828L;

	@Id
	@Column(name = "process_id")
	private String processId;

	@Column(name = "wf_template_id")
	private String wfTemplateId;

	@Column(name = "wf_conf_id")
	private String wfConfigurationId;

	@Column(name = "wf_executor")
	private String wfExecutor;

	@Column(name = "name")
	private String name;

	@Column(name = "family")
	private String family;

	@Column(name = "status")
	@Enumerated(EnumType.STRING)
	private JobStatus status;

	@Column(name = "start_date")
	private LocalDateTime startDate;

	@Column(name = "end_date")
	private LocalDateTime endDate;

	@Column(name = "last_update")
	private LocalDateTime lastUpdate;

	@Column(name = "ds_id")
	private String dsId;

	@Column(name = "ds_name")
	private String dsName;

	@Column(name = "ds_api")
	private String dsApi;

	@Type(JsonType.class)
	@Column(name = "details")
	private Map<String, String> details;

	@PrePersist
	protected void onCreate() {
		lastUpdate = LocalDateTime.now();
	}

	@PreUpdate
	protected void onUpdate() {
		lastUpdate = LocalDateTime.now();
	}

	public String getProcessId() {
		return processId;
	}

	public void setProcessId(final String processId) {
		this.processId = processId;
	}

	public String getWfTemplateId() {
		return wfTemplateId;
	}

	public void setWfTemplateId(final String wfTemplateId) {
		this.wfTemplateId = wfTemplateId;
	}

	public String getWfConfigurationId() {
		return wfConfigurationId;
	}

	public void setWfConfigurationId(final String wfConfigurationId) {
		this.wfConfigurationId = wfConfigurationId;
	}

	public String getWfExecutor() {
		return wfExecutor;
	}

	public void setWfExecutor(final String wfExecutor) {
		this.wfExecutor = wfExecutor;
	}

	public String getName() {
		return name;
	}

	public void setName(final String name) {
		this.name = name;
	}

	public String getFamily() {
		return family;
	}

	public void setFamily(final String family) {
		this.family = family;
	}

	public JobStatus getStatus() {
		return status;
	}

	public void setStatus(final JobStatus status) {
		this.status = status;
	}

	public LocalDateTime getStartDate() {
		return startDate;
	}

	public void setStartDate(final LocalDateTime startDate) {
		this.startDate = startDate;
	}

	public LocalDateTime getEndDate() {
		return endDate;
	}

	public void setEndDate(final LocalDateTime endDate) {
		this.endDate = endDate;
	}

	public LocalDateTime getLastUpdate() {
		return lastUpdate;
	}

	public void setLastUpdate(final LocalDateTime lastUpdate) {
		this.lastUpdate = lastUpdate;
	}

	public String getDsId() {
		return dsId;
	}

	public void setDsId(final String dsId) {
		this.dsId = dsId;
	}

	public String getDsName() {
		return dsName;
	}

	public void setDsName(final String dsName) {
		this.dsName = dsName;
	}

	public String getDsApi() {
		return dsApi;
	}

	public void setDsApi(final String dsApi) {
		this.dsApi = dsApi;
	}

	public Map<String, String> getDetails() {
		return details;
	}

	public void setDetails(final Map<String, String> details) {
		this.details = details;
	}

}
