package eu.dnetlib.wfs.graph;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;

import eu.dnetlib.wfs.procs.Token;

public class Graph {

	private final Map<String, GraphNode> nodes = new HashMap<>();
	private List<Arc> arcs = new ArrayList<>();

	public void addArc(final Arc arc) {
		arcs.add(arc);
	}

	public void addNode(final GraphNode node) {
		nodes.put(node.getName(), node);
	}

	public Set<String> nodeNames() {
		return nodes.keySet();
	}

	public Collection<GraphNode> nodes() {
		return nodes.values();
	}

	public GraphNode getNode(final String name) {
		return nodes.get(name);
	}

	public List<Arc> getArcs() {
		return arcs;
	}

	public void setArcs(final List<Arc> arcs) {
		this.arcs = arcs;
	}

	public Set<GraphNode> startNodes() {
		final Set<GraphNode> res = new HashSet<>();
		for (final GraphNode n : nodes.values()) {
			if (n.isStart()) {
				res.add(n);
			}
		}
		return res;
	}

	public Set<GraphNode> nextNodes(final GraphNode current, final Token token) {
		return arcs.stream()
				.filter(arc -> StringUtils.equals(arc.getFrom(), current.getName()))
				.filter(arc -> arc.isValid(token))
				.map(Arc::getTo)
				.distinct()
				.map(to -> nodes.get(to))
				.collect(Collectors.toSet());

	}

	public int getNumberOfIncomingArcs(final GraphNode node) {
		int count = 0;
		for (final Arc arc : arcs) {
			if (arc.getTo().equals(node.getName())) {
				count++;
			}
		}
		return count;
	}

	@Override
	public String toString() {
		return "\n************************\nNodes: " + nodes + "\nArcs:  " + arcs + "\n************************\n";
	}
}
