package eu.dnetlib.wfs.graph;

import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

public class GraphNode {

	public static final String SUCCESS_NODE = "success";

	private final String name;
	private final String type;
	private final boolean isStart;
	private final boolean isJoin;
	private final boolean isSuccessNode;
	private final Map<String, Object> params;
	private final Map<String, Object> envParams;
	private final Map<String, String> outputEnvMap;

	private GraphNode(final String name,
			final String type,
			final boolean isStart,
			final boolean isJoin,
			final boolean isSuccessNode,
			final Map<String, Object> params,
			final Map<String, Object> envParams,
			final Map<String, String> outputEnvMap) {
		this.name = name;
		this.type = type;
		this.isStart = isStart;
		this.isJoin = isJoin;
		this.isSuccessNode = isSuccessNode;
		this.params = params;
		this.envParams = envParams;
		this.outputEnvMap = outputEnvMap;
	}

	public static GraphNode newNode(final String name,
			final String type,
			final Map<String, Object> params,
			final Map<String, Object> envParams,
			final Map<String, String> outputEnvMap) {
		return new GraphNode(name, type, false, false, false, params, envParams, outputEnvMap);
	}

	public static GraphNode newStartNode(final String name,
			final String type,
			final Map<String, Object> params,
			final Map<String, Object> envParams,
			final Map<String, String> outputEnvMap) {
		return new GraphNode(name, type, true, false, false, params, envParams, outputEnvMap);
	}

	public static GraphNode newJoinNode(final String name,
			final String type,
			final Map<String, Object> params,
			final Map<String, Object> envParams,
			final Map<String, String> outputEnvMap) {
		return new GraphNode(name, type, false, true, false, params, envParams, outputEnvMap);
	}

	public static GraphNode newSuccessNode() {
		return new GraphNode(SUCCESS_NODE, null, false, true, true, new HashMap<>(), new HashMap<>(), new HashMap<>());
	}

	public String getName() {
		return name;
	}

	public String getType() {
		return type;
	}

	public boolean isStart() {
		return isStart;
	}

	public boolean isJoin() {
		return isJoin;
	}

	public boolean isSuccessNode() {
		return isSuccessNode;
	}

	@Override
	public String toString() {
		final StringWriter sw = new StringWriter();
		sw.append("[ name: ");
		sw.append(name);
		if (StringUtils.isNotBlank(type)) {
			sw.append(", type: ");
			sw.append(type);
		}
		if (isStart()) {
			sw.append(" - isStart");
		}
		if (isJoin()) {
			sw.append(" - isJoin");
		}
		sw.append(" ]");
		return sw.toString();
	}

	public Map<String, Object> getParams() {
		return params;
	}

	public Map<String, Object> getEnvParams() {
		return envParams;
	}

	public Map<String, String> getOutputEnvMap() {
		return outputEnvMap;
	}

}
