package eu.dnetlib.wfs.repository;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.transaction.annotation.Transactional;

import eu.dnetlib.domain.wfs.JobStatus;
import eu.dnetlib.domain.wfs.WfJournalEntry;

public interface WfJournalEntryRepository extends JpaRepository<WfJournalEntry, String> {

	// IMPORTANT: Postgres, by default, should return nulls first when you specify 'ORDER BY DESC'.
	// Otherwise it will be necessary to implement the native queries specifying 'ORDER BY end_date DESC NULLS FIRST';

	Page<WfJournalEntry> findByLastUpdateBetweenOrderByLastUpdateDesc(LocalDateTime start, LocalDateTime end, Pageable pageable);

	Page<WfJournalEntry> findByWfConfigurationIdOrderByLastUpdateDesc(String id, Pageable pageable);

	Page<WfJournalEntry> findAllByOrderByLastUpdateDesc(Pageable pageable);

	Page<WfJournalEntry> findByDsIdOrderByLastUpdateDesc(String dsId, Pageable pageable);

	Page<WfJournalEntry> findByDsApiOrderByLastUpdateDesc(String apiId, Pageable pageable);

	List<WfJournalEntry> findByStatus(JobStatus status);

	Optional<WfJournalEntry> findFirstByWfConfigurationIdAndEndDateNotNullOrderByEndDateDesc(String id);

	@Modifying
	@Transactional
	@Query(value = "update WfJournalEntry set wfExecutor = ?2, status = ?3, startDate = ?4, lastUpdate = ?4 where processId = ?1 and wfExecutor is NULL")
	void tryAssegnment(String id, String workerId, JobStatus status, LocalDateTime startDate);

}
