package eu.dnetlib.wfs.utils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import eu.dnetlib.domain.wfs.WfConfiguration;
import eu.dnetlib.domain.wfs.WfTemplate.WfParam;

public class WfConfigurationUtils {

	private static final Log log = LogFactory.getLog(WfConfigurationUtils.class);

	public static boolean isConfigured(final List<WfParam> wfTemplateParams, final WfConfiguration conf) {

		final Map<String, Object> confParams = allConfiguredParameters(wfTemplateParams, conf);

		final long count = wfTemplateParams.stream()
				.filter(WfParam::isRequired)
				.filter(p -> StringUtils.isNotBlank(p.getDefaultValue()) || (confParams.get(p.getName()) != null))
				.count();

		if (log.isDebugEnabled()) {
			log.debug(String.format("WfConf %s not configured: there are %s required parameters without value", conf.getId(), count));
		}

		return count == 0;
	}

	public static Map<String, Object> allConfiguredParameters(final List<WfParam> wfTemplateParams, final WfConfiguration conf) {
		final Map<String, Object> all = new HashMap<>();

		if (wfTemplateParams != null) {
			wfTemplateParams.stream()
					.filter(p -> StringUtils.isNotBlank(p.getDefaultValue()))
					.forEach(p -> all.put(p.getName(), p.getDefaultValue()));
		}
		if (conf.getUserParams() != null) {
			all.putAll(conf.getSystemParams());
		}
		if (conf.getSystemParams() != null) {
			all.putAll(conf.getUserParams());
		}
		if (StringUtils.isNotBlank(conf.getDsId())) {
			all.put("dsId", conf.getDsId());
		}
		if (StringUtils.isNotBlank(conf.getDsName())) {
			all.put("dsName", conf.getDsName());
		}
		if (StringUtils.isNotBlank(conf.getApiId())) {
			all.put("apiId", conf.getApiId());
		}

		if (log.isDebugEnabled()) {
			log.debug("** CONFIGURED INPUT WF PARAMETERS **");
			all.forEach((k, v) -> log.debug("* " + k + ": " + v));
			log.debug("************************************");

		}

		return all;
	}

}
