package eu.dnetlib.wfs.nodes.conf;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;

import eu.dnetlib.common.clients.DnetServiceClientFactory;
import eu.dnetlib.common.clients.SimpleResourceClient;
import eu.dnetlib.domain.dsm.Api;
import eu.dnetlib.domain.dsm.Datasource;
import eu.dnetlib.domain.wfs.WfConfiguration;
import eu.dnetlib.domain.wfs.WfTemplate;
import eu.dnetlib.domain.wfs.WfTemplate.WfParam;
import eu.dnetlib.domain.wfs.WorkflowsConstants;
import eu.dnetlib.wfs.annotations.WfInputParam;
import eu.dnetlib.wfs.annotations.WfNode;
import eu.dnetlib.wfs.annotations.WfOutputParam;
import eu.dnetlib.wfs.nodes.AbstractJobNode;
import eu.dnetlib.wfs.repository.WfConfigurationRepository;
import eu.dnetlib.wfs.utils.WfConfigurationUtils;

@WfNode("registerWfConfiguration")
public class RegisterWfConfigurationNode extends AbstractJobNode {

	private static final int DEFAULT_AGGR_PRIORITY = 75;

	@WfInputParam
	private String wfId;

	@WfInputParam
	private String destroyWfId;

	@WfInputParam
	private Datasource ds;

	@WfInputParam
	private Api api;

	@WfInputParam
	private String nativeMdStoreId;

	@WfInputParam
	private String cleanedMdStoreId;

	@WfOutputParam
	private String wfConfId;

	@Autowired
	private WfConfigurationRepository wfConfigurationRepository;

	@Autowired
	private DnetServiceClientFactory clientFactory;

	@Override
	protected void execute() throws Exception {

		wfConfId = "wf-aggr-" + UUID.randomUUID();

		final WfConfiguration conf = new WfConfiguration();
		conf.setId(wfConfId);
		conf.setSection(WorkflowsConstants.AGGREGATION_WF_CONFS_SECTION);
		conf.setParentId(null);

		conf.setName(String.format("Aggregation of '%s' (api: %s) ", ds.getOfficialname(), api.getId()));
		conf.setDetails(new HashMap<>());

		conf.setDsId(ds.getId());
		conf.setDsName(ds.getOfficialname());
		conf.setApiId(api.getId());

		conf.setWorkflow(wfId);
		conf.setDestroyWf(destroyWfId);
		conf.setEnabled(true);
		conf.setPriority(DEFAULT_AGGR_PRIORITY);

		conf.setSchedulingEnabled(false);
		conf.setCronExpression("0 0 0 ? * *");
		conf.setCronMinInterval(10080);

		conf.setSystemParams(prepareSystemParams());

		final List<WfParam> wfParams = clientFactory.getClient(SimpleResourceClient.class)
				.findResourceContent(WorkflowsConstants.WF_TEMPLATE, conf.getWorkflow(), WfTemplate.class)
				.getParameters();

		conf.setConfigured(WfConfigurationUtils.isConfigured(wfParams, conf));

		wfConfigurationRepository.save(conf);
	}

	private Map<String, Object> prepareSystemParams() {
		final Map<String, Object> map = new LinkedHashMap<>();
		map.put("nativeMdStoreId", nativeMdStoreId);
		map.put("cleanedMdStoreId", cleanedMdStoreId);
		return map;
	}

}
