package eu.dnetlib.manager.wf.model;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.util.Arrays;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import eu.dnetlib.domain.wfs.WfTemplate;
import eu.dnetlib.domain.wfs.WfTemplate.Arc;
import eu.dnetlib.domain.wfs.WfTemplate.Node;
import eu.dnetlib.domain.wfs.WfTemplate.NodeInputParam;
import eu.dnetlib.domain.wfs.WfTemplate.WfParam;
import eu.dnetlib.domain.wfs.WfTemplate.WfParamType;

class WorkflowTemplateTest {

	private final ObjectMapper mapper = new ObjectMapper();

	private WfTemplate tmpl;

	private WfParam p1;
	private WfParam p2;
	private WfParam p3;

	private Node n1;
	private Node n2;
	private Node n3;

	private NodeInputParam np1;

	private Arc a12;
	private Arc a13;
	private Arc a23;
	private Arc a3succ;

	@BeforeEach
	private void setUp() throws Exception {

		a12 = new Arc("N2");
		a13 = new Arc("N3");
		a23 = new Arc("N3");
		a3succ = new Arc("success");

		p1 = new WfParam("Name");
		p2 = new WfParam("Age", WfParamType.NUMBER);
		p3 = new WfParam("Birthday", "Your Birthday", WfParamType.DATE, "1900-01-01", false);

		np1 = NodeInputParam.newRefParam("Age", "Age");
		np1 = NodeInputParam.newSimpleParam("Test", "test");

		n1 = new Node("N1", true);
		n1.setArcs(Arrays.asList(a12, a13));

		n2 = new Node("N2", "T2");
		n2.setArcs(Arrays.asList(a23));
		n2.setInput(Arrays.asList(np1));

		n3 = new Node("N3");
		n2.setArcs(Arrays.asList(a3succ));

		tmpl = new WfTemplate();
		tmpl.setParameters(Arrays.asList(p1, p2, p3));
		tmpl.setGraph(Arrays.asList(n1, n2, n3));
	}

	@Test
	final void test() throws JsonProcessingException {
		final String json = mapper.writeValueAsString(tmpl);

		System.out.println(json);

		final WfTemplate tmpl2 = mapper.readValue(json, WfTemplate.class);

		assertEquals(tmpl.getGraph().size(), tmpl2.getGraph().size());
		assertEquals(tmpl.getParameters().size(), tmpl2.getParameters().size());
	}

}
