package eu.dnetlib.manager.wf.workflows.graph;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.function.Function;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.context.expression.MapAccessor;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;

import eu.dnetlib.wfs.procs.Token;

public class GraphLoaderTest {

	private Token token;

	@BeforeEach
	void setUp() throws Exception {
		token = new Token();
		token.setEnvAttribute("author", "Michele Artini");
		token.setEnvAttribute("age", 47);
	}

	@Test
	final void testExpressions() {
		assertTrue(evalFunction("age == 47").apply(token));
		assertTrue(evalFunction("age > 40").apply(token));
		assertTrue(evalFunction("author == 'Michele Artini'").apply(token));
		assertTrue(evalFunction("age == 47 && author == 'Michele Artini'").apply(token));
		assertTrue(evalFunction("age == 47 || author == 'Michele Artini'").apply(token));
		assertTrue(evalFunction("age == 47 || author == 'Claudio Atzori'").apply(token));
		assertTrue(evalFunction("age == 22 || author == 'Michele Artini'").apply(token));
		assertFalse(evalFunction("age != 47").apply(token));
		assertFalse(evalFunction("age < 40").apply(token));
		assertFalse(evalFunction("author != 'Michele Artini'").apply(token));
		assertFalse(evalFunction("age == 47 && author == 'Claudio Atzori'").apply(token));
	}

	private Function<Token, Boolean> evalFunction(final String f) {
		return token -> {
			final ExpressionParser parser = new SpelExpressionParser();

			final StandardEvaluationContext context = new StandardEvaluationContext(token.getEnv());
			context.addPropertyAccessor(new MapAccessor());

			return parser.parseExpression(f).getValue(context, Boolean.class);
		};
	}

}
